/*
 *	DashBox Dashboard Widget
 *	
 *	Copyright 2005 Max Antoni
 *	http://www.maxantoni.de
 */
var canvasWidth = 220;
var canvasHeight = 260;
var canvasOffsetLeft = 10;
var canvasOffsetTop = 5;
var edgeRadius = 5;

var URL_HOMEPAGE = "http://www.maxantoni.de";

var currentLevel = 0;
var defaultLevelCount = 0;
var messageElement = null;
var levelBackup = null;

function showMessage(n, m, cls) {
	window.onkeydown = function(){};
	while(n.hasChildNodes())
		n.removeChild(n.lastChild);
	n.className = cls;
	n.appendChild(m);
	n.style.opacity = "0.1";
	n.style.visibility = "visible";
	messageElement = n;
	setTimeout("showMessageAni()", 30);
}

function showMessageAni() {
	var o = Number(messageElement.style.opacity) + 0.05;
	messageElement.style.opacity = String(o);
	if(o < 0.9) {
		setTimeout("showMessageAni()", 30);
	}
}

function hideMessage() {
	if(messageElement) {
		messageElement.style.visibility = "hidden";
		while(messageElement.hasChildNodes())
			messageElement.removeChild(messageElement.lastChild);
		if(messageElement == messageFront)
			BOARD.restoreFocus();
		messageElement = null;
	}
}

function simpleMessage(p, m, cls) {
	var n = document.createElement("span");
	n.appendChild(document.createTextNode(m));
	n.appendChild(document.createElement("br"));
	n.appendChild(document.createElement("br"));
	var s = document.createElement("input");
	s.setAttribute("type", "submit");
	s.value = "OK";
	s.onclick = function() {
		hideMessage();
	};
	n.appendChild(s);
	showMessage(p, n, cls);
}

function showHelp() {
	var n = document.createElement("span");
	n.appendChild(document.createTextNode("Use the arrow keys to push the white boxes onto the orange targets."));
	n.appendChild(document.createElement("br"));
	n.appendChild(document.createTextNode("You can only push one box at a time."));
	var ul = document.createElement("ul");
	var li = document.createElement("li");
	li.appendChild(document.createTextNode("left - turn left"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("right - turn right"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("up - move forward / push"));
	ul.appendChild(li);
	n.appendChild(ul);
	var s = document.createElement("input");
	s.setAttribute("type", "submit");
	s.value = "OK";
	s.onclick = function() {
		hideMessage();
	};
	n.appendChild(s);
	showMessage(messageFront, n, "instructionsMessage");
}

function showEditorHelp() {
	var n = document.createElement("span");
	n.appendChild(document.createTextNode("Use your keyboard:"));
	var ul = document.createElement("ul");
	var li = document.createElement("li");
	li.appendChild(document.createTextNode("p - player position"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("w - wall"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("t - tower"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("b - box"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("x - target"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("i - invisible box"));
	ul.appendChild(li);
	li = document.createElement("li");
	li.appendChild(document.createTextNode("backspace - delete"));
	ul.appendChild(li);
	n.appendChild(ul);
	var s = document.createElement("input");
	s.setAttribute("type", "submit");
	s.value = "OK";
	s.onclick = function() {
		hideMessage();
	};
	n.appendChild(s);
	showMessage(messageFront, n, "instructionsMessage");
}

function performAction(action) {
	hideMessage();
	switch(action) {	
	case "restart":
		break;
	case "new":
		currentLevel = ALL_LEVELS.length;
		var f = new Array();
		for(var y=0;y<MAX_LEVEL_HEIGHT;y++) {
			f[y] = new Array(MAX_LEVEL_WIDTH);
		}
		ALL_LEVELS.push(new Level("New Level", f, Math.floor(MAX_LEVEL_WIDTH/2.0), Math.floor(MAX_LEVEL_HEIGHT/2.0), true));
		playTools.style.visibility = "hidden";
		editTools.style.visibility = "visible";
		levelName.value = "New Level";
		levelSelect.options[levelSelect.options.length] = new Option("New Level", String(currentLevel), false, true);
		break;
	case "edit":
		var f = new Array();
		for(var y=0;y<MAX_LEVEL_HEIGHT;y++) {
			f[y] = new Array(MAX_LEVEL_WIDTH);
		}
		var px = Math.floor(MAX_LEVEL_WIDTH/2.0) - Math.floor(BOARD.maxX/2.0);
		var py = Math.floor(MAX_LEVEL_HEIGHT/2.0) - Math.floor(BOARD.fields.length/2.0);
		for(var y=0;y<BOARD.fields.length;y++) {
			for(var x=0;x<BOARD.maxX;x++) {
				f[py+y][px+x] = ALL_LEVELS[currentLevel].fields[y][x];
			}
		}
		if(currentLevel < defaultLevelCount) {
			ALL_LEVELS.push(new Level("Copy of " + ALL_LEVELS[currentLevel].name, f, ALL_LEVELS[currentLevel].playerX+px, ALL_LEVELS[currentLevel].playerY+py, true));
			currentLevel = ALL_LEVELS.length-1;
			levelSelect.options[levelSelect.options.length] = new Option("New Level", String(currentLevel), false, true);
		}
		else {
			levelBackup = ALL_LEVELS[currentLevel].toString();
			ALL_LEVELS[currentLevel].fields = f;
			ALL_LEVELS[currentLevel].playerX += px;
			ALL_LEVELS[currentLevel].playerY += py;
			ALL_LEVELS[currentLevel].edit = true;
		}
		BOARD.setLevel(ALL_LEVELS[currentLevel]);
		BOARD.draw();
		playTools.style.visibility = "hidden";
		editTools.style.visibility = "visible";
		levelName.value = ALL_LEVELS[currentLevel].name;
		break;
	case "remove":
		if(currentLevel < defaultLevelCount) {
			simpleMessage(messageFront, "You cannot delete a build-in Level.", "levelCompleteMessage");
			actionSelect.selectedIndex = 0;
			return;
		}
		var n = document.createElement("span");
		n.appendChild(document.createTextNode("Realy delete \"" + ALL_LEVELS[currentLevel].name + "\" ?"));
		n.appendChild(document.createElement("br"));
		n.appendChild(document.createElement("br"));
		var s1 = document.createElement("input");
		s1.setAttribute("type", "submit");
		s1.value = "Cancel";
		s1.onclick = function() {
			hideMessage();
		};
		n.appendChild(s1);
		var s2 = document.createElement("input");
		s2.setAttribute("type", "submit");
		s2.value = "OK";
		s2.onclick = function() {
			hideMessage();
			ALL_LEVELS.splice(currentLevel, 1);
			var i = currentLevel;
			while(getPref("level_" + i, null) != null) {
				setPref("level_" + i, getPref("level_" + (i+1), null));
				i++;
			}
			levelSelect.removeChild(levelSelect.childNodes[currentLevel]);
			if(currentLevel == ALL_LEVELS.length) {
				currentLevel = 0;
			}
			levelSelect.selectedIndex = currentLevel;
			setTitle(ALL_LEVELS[currentLevel].name);
			BOARD.setLevel(ALL_LEVELS[currentLevel]);
			BOARD.draw();
		};
		n.appendChild(s2);
		showMessage(messageFront, n, "levelCompleteMessage");
		actionSelect.selectedIndex = 0;
		return;
	default:
		actionSelect.selectedIndex = 0;
		return;
	}
	actionSelect.selectedIndex = 0;
	setTitle(ALL_LEVELS[currentLevel].name);
	BOARD.setLevel(ALL_LEVELS[currentLevel]);
	BOARD.draw();
}

function editCancel() {
	hideMessage();
	if(levelBackup) {
		ALL_LEVELS[currentLevel] = Level.fromString(levelBackup);
		levelBackup = null;
	}
	else {
		levelSelect.removeChild(levelSelect.childNodes[currentLevel]);
		ALL_LEVELS.splice(currentLevel, 1);
		if(currentLevel == ALL_LEVELS.length) {
			currentLevel = 0;
		}
		levelSelect.selectedIndex = currentLevel;
	}
	editTools.style.visibility = "hidden";
	playTools.style.visibility = "visible";
	setTitle(ALL_LEVELS[currentLevel].name);
	BOARD.setLevel(ALL_LEVELS[currentLevel]);
	BOARD.draw();
}

function editDone() {
	levelBackup = null;
	hideMessage();
	var l = ALL_LEVELS[currentLevel];
	var boxCount = 0;
	var targetCount = 0;
	for(var y=0;y<MAX_LEVEL_HEIGHT;y++) {
		for(var x=0;x<MAX_LEVEL_WIDTH;x++) {
			if(l.fields[y][x] != null) {
				if(l.fields[y][x] == B) {
					boxCount++;
				}
				else if(l.fields[y][x] == X) {
					targetCount++;
				}
			}
		}
	}
	if(boxCount == 0 || targetCount == 0) {
		simpleMessage(messageFront, "There must be at least one box and one target.", "levelCompleteMessage");
		return;
	}
	if(boxCount != targetCount) {
		simpleMessage(messageFront, "Number of boxes and targets not equal.", "levelCompleteMessage");
		return;
	}

	var f = new Array();
	var b = MAX_LEVEL_WIDTH;
	for(var y=0;y<MAX_LEVEL_HEIGHT;y++) {
		var fnd = false;
		for(var x=0;x<MAX_LEVEL_WIDTH;x++) {
			if(l.fields[y][x] != null) {
				if(f.length == 0)
					l.playerY -= y;
				f.push(l.fields[y]);
				if(x < b)
					b = x;
				fnd = true;
				break;
			}
		}
	}
	b = Math.min(b, l.playerX);
	l.playerX -= b;
	for(var y=0;y<f.length;y++) {
		var e = 0;
		for(var x=MAX_LEVEL_WIDTH-1;x>=0;x--) {
			if(f[y][x] != null) {
				e = x;
				break;
			}
		}
		f[y] = f[y].slice(b, e+1);
	}
	l.fields = f;
	l.name = levelName.value;
	l.edit = false;
	ALL_LEVELS[currentLevel] = l;
	setPref("level_" + currentLevel, l.toString());
	with(levelSelect.options[currentLevel]) {
		removeChild(firstChild);
		appendChild(document.createTextNode(l.name));
	}
	editTools.style.visibility = "hidden";
	playTools.style.visibility = "visible";
	setTitle(ALL_LEVELS[currentLevel].name);
	BOARD.setLevel(ALL_LEVELS[currentLevel]);
	BOARD.draw();
}

function setTitle(t) {
	title.removeChild(title.firstChild);
	var n = document.createTextNode("DashBox - " + t);
	title.appendChild(n);
	while(n.offsetWidth > 200) {
		n = document.createTextNode(n.nodeValue.substring(0, n.nodeValue.length - 4) + "...");
		title.replaceChild(n, title.firstChild);
	}
	title.setAttribute("title", t);
}

function jumpToLevel(l) {
	hideMessage();
	currentLevel = Number(l);
	setPref("level", String(currentLevel));
	setTitle(ALL_LEVELS[currentLevel].name);
	levelSelect.selectedIndex = currentLevel;
	BOARD.setLevel(ALL_LEVELS[currentLevel]);
	BOARD.draw();
}

function loadLevel(s) {
	window.onkeydown = function(){};
	_loadLevel(s);
}

function _loadLevel(s) {
	var l = Level.fromString(s);
	if(l == null) {
		simpleMessage(messageBack, "Not a valid Level-Code!", "levelCompleteMessage");
		return;
	}
	for(var i=0;i<ALL_LEVELS.length;i++) {
		if(ALL_LEVELS[i].name == l.name) {
			l.name = "";
			var m = document.createElement("span");
			m.appendChild(document.createTextNode("\"" + ALL_LEVELS[i].name + "\" already exists."));
			m.appendChild(document.createElement("br"));
			m.appendChild(document.createTextNode("New name:"));
			var ip = document.createElement("input");
			ip.setAttribute("type", "text");
			ip.value = ALL_LEVELS[i].name;
			m.levelcode = l.toString();
			ip.onkeydown = function(e) {
				if(event.keyCode==13) {
					// Crashes Safari, but works in Dashboard. D'oh!
					_loadLevel(this.value + this.parentNode.levelcode);
				}
			};
			m.appendChild(ip);
			var cl = document.createElement("input");
			cl.setAttribute("type", "submit");
			cl.value = "Cancel";
			cl.onclick = function() {
				hideMessage();
			};
			m.appendChild(cl);
			var ok = document.createElement("input");
			ok.setAttribute("type", "submit");
			ok.value = "OK";
			ok.onclick = function() {
				_loadLevel(this.parentNode.childNodes[3].value + this.parentNode.levelcode);
			};
			m.appendChild(ok);
			showMessage(messageBack, m, "levelCompleteMessage");
			return;
		}
	}
	messageBack.style.visibility = "hidden";
	ALL_LEVELS.push(l);
	levelSelect.options[levelSelect.options.length] = new Option(l.name, String(ALL_LEVELS.length-1), false, true);
	setPref("level_" + (ALL_LEVELS.length-1), s);
	jumpToLevel(ALL_LEVELS.length-1);
	showFront();
}

function initWidget() {
	messageFront.style.visibility = "hidden";
	messageBack.style.visibility = "hidden";
	drawFront();
	
	createGenericButton(actionButton, "Action", function(){});
	createGenericButton(levelButton, "Level", function(){});
	createGenericButton(helpButton, "?", showHelp);

	createGenericButton(doneButton, "Done", showFront);

	createGenericButton(editHelpButton, "?", showEditorHelp);
	createGenericButton(editCancelButton, "X", editCancel);
	createGenericButton(editDoneButton, "Done", editDone);

	
	defaultLevelCount = ALL_LEVELS.length;
	while(true) {
		var s = getPref("level_" + ALL_LEVELS.length, null);
		if(s == null)
			break;
		ALL_LEVELS[ALL_LEVELS.length] = Level.fromString(s);
	}
	for(var i=0; i< ALL_LEVELS.length; i++) {
		levelSelect.options[levelSelect.options.length] = new Option(ALL_LEVELS[i].name, String(i), false, i == currentLevel);
	}
	currentLevel = Number(getPref("level", "0"));
	levelSelect.selectedIndex = currentLevel;
	initGame(ALL_LEVELS[currentLevel]);
	setTitle(ALL_LEVELS[currentLevel].name);
}

function drawFront() {
    back.style.visibility = "hidden";
    front.style.visibility = "visible";
    var context = frameFront.getContext("2d");
    draw(context);
}

function drawBack() {
    front.style.visibility = "hidden";
    back.style.visibility = "visible";
    var context = frameBack.getContext("2d");
    draw(context);
}

function draw(context) {
	context.clearRect(0, 0, canvasWidth + canvasOffsetLeft * 2,
						canvasHeight + canvasOffsetTop * 2 + canvasHeight);

	createPath(context);
	context.save();
	context.setFillColor(0.0, 0.0, 0.0, 1.0);
	context.setShadow(0, 7, 10, 0.0, 0.5);
	context.fill();
	context.restore();
	
	context.save();
	createPath(context);
	context.fillStyle = context.createLinearGradient(0,0,0,canvasHeight);
	context.fillStyle.addColorStop(0, "#eee");
	context.fillStyle.addColorStop(0.07, "#888");
	context.fillStyle.addColorStop(0.5, "#555");
	context.fillStyle.addColorStop(0.51, "#222");
	context.fillStyle.addColorStop(0.91, "#333");
	context.fillStyle.addColorStop(1, "#444");
	context.fill();
	context.restore();

	context.save();
	createPath(context);
	context.setStrokeColor("#222222", 1.0);
	context.setLineWidth(1.5);
	context.setLineCap("round");
	context.stroke();
	context.restore();
}

function createPath(context) {
	context.beginPath();
	context.moveTo(canvasOffsetLeft + edgeRadius, canvasOffsetTop);
	context.arcTo(canvasOffsetLeft + canvasWidth, canvasOffsetTop,
					canvasOffsetLeft + canvasWidth, canvasOffsetTop + edgeRadius,
					edgeRadius);
	context.arcTo(canvasOffsetLeft + canvasWidth, canvasOffsetTop + canvasHeight,
					canvasOffsetLeft + canvasWidth - edgeRadius, canvasOffsetTop + canvasHeight,
					edgeRadius);
	context.arcTo(canvasOffsetLeft, canvasOffsetTop + canvasHeight,
					canvasOffsetLeft, canvasOffsetTop + canvasHeight - edgeRadius,
					edgeRadius);
	context.arcTo(canvasOffsetLeft, canvasOffsetTop,
					canvasOffsetLeft + edgeRadius, canvasOffsetTop,
					edgeRadius);
	context.closePath();
}

function showMaxAntoni() {
	if(window.widget) {
		widget.openURL(URL_HOMEPAGE);
	}
	else {
		alert(URL_HOMEPAGE);
	}
}

function showBack() {
	hideMessage();
	exitflip();
	window.onkeydown = function(){};
    if(window.widget)
        widget.prepareForTransition("ToBack");       
    drawBack();
    animations.checked = ANIMATIONS;
    levelInput.value = ALL_LEVELS[currentLevel].toString();
    if(window.widget)
        setTimeout('widget.performTransition();', 0);
}

function showFront() {
	hideMessage();
    if(window.widget)
        widget.prepareForTransition("ToFront");
    drawFront();
	if(animations.checked != ANIMATIONS) {
		ANIMATIONS = animations.checked;
		setPref("animations", ANIMATIONS ? "true" : "false");
	}
	settingsUpdate();
    if(window.widget)
        setTimeout('widget.performTransition();', 0);
}

function getPref(key, def) {
	if(window.widget) {
    	var value = widget.preferenceForKey(key);
    	if(value) {
        	return value;
    	}
	}
	return def;
}

function setPref(key, value) {
	if(window.widget) {
		widget.setPreferenceForKey(value, key);
	}
}

