/*
 *	DashBox Dashboard Widget
 *	
 *	Copyright 2005 Max Antoni
 *	http://www.maxantoni.de
 */
var BLOCK_SIZE = 30;
var BLOCK_SIZE_HALF = BLOCK_SIZE / 2;

var NORTH = 1
var EAST = 2
var SOUTH = 3
var WEST = 4

var MAX_LEVEL_WIDTH = 9;
var MAX_LEVEL_HEIGHT = 9;

var ANIMATIONS = getPref("animations", "true") == "true";

function settingsUpdate() {
	ROTATION_SPEED = ANIMATIONS ? 30 : 90;
	BOARD.restoreFocus();
}
ROTATION_SPEED = ANIMATIONS ? 30 : 90;


var B = 1; // Box
var X = 2; // Target
var W = 3; // Wall
var T = 4; // Tower
var I = 6; // Invisible

var BOX_COLORS = ["#fff", "#888", "#ddd", "#888", "#ddd", "#888", "#eee", "#888", "#ccc", "#888"];
var WALL_COLORS = ["#9bf", "#333", "#7ae", "#333", "#7ae", "#333", "#cdf", "#333", "#69e", "#333"];
var INVISIBLE_COLORS = ["#999", "#555", "#999", "#555", "#999", "#555", "#999", "#555", "#999", "#555"];

var BOARD = null;

function Box() {
	this.onTarget = false;
}
Box.prototype.draw = function(context, x, y) {
	if(this.onTarget) {
		Target.prototype.draw(context, x, y);
	}
	context.globalAlpha = 0.75;
	BOARD.block(context, x, y, 0.5, BOX_COLORS);
	context.globalAlpha = 1.0;
};


function Wall(dp) {
	this.dp = dp;
}
Wall.prototype.draw = function(context, x, y) {
	if(this.dp == 0) {
		if(BOARD.edit)
			BOARD.block(context, x, y, 0.3, INVISIBLE_COLORS);
	}
	else {
		BOARD.block(context, x, y, this.dp, WALL_COLORS);
	}
};

function Target() {
}
Target.prototype.draw = function(context, x, y) {
	var rx = BOARD.offsetLeft + x * BLOCK_SIZE;
	var ry = BOARD.offsetTop + y * BLOCK_SIZE;
	context.fillStyle = "#fa4";
	context.fillRect(rx, ry, BLOCK_SIZE, BLOCK_SIZE);
	context.setStrokeColor("#a52", 1.0);
	context.setLineWidth(2);
	context.setLineCap("round");
	context.strokeRect(rx, ry, BLOCK_SIZE, BLOCK_SIZE);
};

function showCompleted() {
	BOARD.zoom -= 0.05;
	BOARD.draw();
	if(BOARD.zoom > 0.4) {
		setTimeout("showCompleted()", 30);
	}
	else {
		setTimeout("showCompletedMessage()", 500);
	}
}

function showCompletedMessage() {
	var m = document.createElement("span");
	m.appendChild(document.createTextNode("Complete!"));
	m.appendChild(document.createElement("br"));
	m.appendChild(document.createElement("br"));
	m.appendChild(document.createTextNode("Steps: " + BOARD.player.steps));
	m.appendChild(document.createElement("br"));
	m.appendChild(document.createTextNode("Time: " + BOARD.player. time + " s"));
	m.appendChild(document.createElement("br"));
	m.appendChild(document.createElement("br"));
	m.appendChild(document.createTextNode("Press space"));
	showMessage(messageFront, m, "levelCompleteMessage");
	window.onkeydown = function(e) {
		if(e.keyCode == 32) {
			e.preventDefault();
			var l = currentLevel + 1;
			if(l >= ALL_LEVELS.length) l = 0;
			jumpToLevel(l);
		}
	};
}

function Player(x, y) {
	this.x = x;
	this.y = y;
	this.dir = NORTH;
	this.nx = 0;
	this.ny = 0;
	this.timer = null;
	this.time = new Date().getTime();
	this.steps = 0;
}
Player.prototype.stopTime = function() {
	this.time = Math.round((new Date().getTime() - this.time) / 100) / 10;
}
Player.prototype.turnLeft = function() {
	switch(this.dir) {
	case NORTH: this.dir = WEST; break;
	case SOUTH: this.dir = EAST; break;
	case WEST: this.dir = SOUTH; break;
	case EAST: this.dir = NORTH; break;
	}
	this.nx = 0;
	this.ny = 0;
	if(this.timer) {
		clearInterval(this.timer);
		this.timer = null;
	}
};
Player.prototype.turnRight = function() {
	switch(this.dir) {
	case NORTH: this.dir = EAST; break;
	case SOUTH: this.dir = WEST; break;
	case WEST: this.dir = NORTH; break;
	case EAST: this.dir = SOUTH; break;
	}
	this.nx = 0;
	this.ny = 0;
	if(this.timer) {
		clearInterval(this.timer);
		this.timer = null;
	}
};
Player.prototype.testField = function(y, x) {
	if(x < 0 || y < 0 || y > BOARD.fields.length - 1 || x > BOARD.fields[y].length)
		return false;
	var f = BOARD.fields[y][x];
	return f == null || f instanceof Target;
};
Player.prototype.moveElement = function(sy, sx, ty, tx) {
	if(BOARD.fields[sy][sx] instanceof Box) {
		var restoreTarget = false;
		if(BOARD.fields[sy][sx].onTarget) {
			BOARD.fields[sy][sx].onTarget = false;
			restoreTarget = true;
		}
		if(BOARD.fields[ty][tx] instanceof Target) {
			BOARD.fields[sy][sx].onTarget = true;
		}
		BOARD.fields[ty][tx] = BOARD.fields[sy][sx];
		BOARD.fields[sy][sx] = restoreTarget ? new Target() : null;
		
		for(var y =0; y<BOARD.fields.length; y++) {
			for(var x =0; x<BOARD.fields[y].length; x++) {
				var f = BOARD.fields[y][x];
				if(f != null) {
					if(f instanceof Target || (f instanceof Box && !f.onTarget))
						return;
				}
			}
		}
		
		// Level complete:
		window.onkeydown = function(){};
		this.stopTime();
		BOARD.zoom = 1;
		setTimeout("showCompleted()", 30);
	}
};
Player.prototype.move = function() {
	switch(this.dir) {
	case NORTH:
		var f = this.y > 0 ? BOARD.fields[this.y-1][this.x] : null;
		if(this.y > 0 && (f == null || f instanceof Target || (f instanceof Box && this.testField(this.y-2, this.x)))) {
			this.y--;
			this.steps++;
			if(ANIMATIONS) this.ny += BLOCK_SIZE_HALF;
			if(f != null) this.moveElement(this.y, this.x, this.y-1, this.x);
		}
		else {
			if(ANIMATIONS) this.ny -= BLOCK_SIZE_HALF;
		}
		BOARD.draw();
		break;
	case SOUTH:
		var f = this.y < BOARD.fields.length - 1 ? BOARD.fields[this.y+1][this.x] : null;
		if(this.y < BOARD.fields.length - 1 && (f == null || f instanceof Target || (f instanceof Box && this.testField(this.y+2, this.x)))) {
			this.y++;
			this.steps++;
			if(ANIMATIONS) this.ny -= BLOCK_SIZE_HALF;
			if(f != null) this.moveElement(this.y, this.x, this.y+1, this.x);
		}
		else {
			if(ANIMATIONS) this.ny += BLOCK_SIZE_HALF;
		}
		BOARD.draw();
		break;
	case EAST:
		var f = this.x > 0 ? BOARD.fields[this.y][this.x-1] : null;
		if(this.x > 0 && (f == null || f instanceof Target || (f instanceof Box && this.testField(this.y, this.x-2)))) {
			this.x--;
			this.steps++;
			if(ANIMATIONS) this.nx += BLOCK_SIZE_HALF;
			if(f != null) this.moveElement(this.y, this.x, this.y, this.x-1);
		}
		else {
			if(ANIMATIONS) this.nx -= BLOCK_SIZE_HALF;
		}
		BOARD.draw();
		break;
	case WEST:
		var f = this.x < BOARD.fields[this.y].length - 1 ? BOARD.fields[this.y][this.x+1] : null;
		if(this.x < BOARD.fields[this.y].length - 1 && (f == null || f instanceof Target || (f instanceof Box && this.testField(this.y, this.x+2)))) {
			this.x++;
			this.steps++;
			if(ANIMATIONS) this.nx -= BLOCK_SIZE / 2;
			if(f != null) this.moveElement(this.y, this.x, this.y, this.x+1);
		}
		else {
			if(ANIMATIONS) this.nx += BLOCK_SIZE / 2;
		}
		BOARD.draw();
		break;
	}
	if(ANIMATIONS && !this.timer)
		this.timer = setInterval("BOARD.player.swing()", 25);
};
Player.prototype.swing = function() {
	if(this.nx < -1 || this.nx > 1) this.nx /= 2.0;
	else this.nx = 0;
	if(this.ny < -1 || this.ny > 1) this.ny /= 2.0;
	else this.ny = 0;
	if(this.nx === 0 && this.ny === 0 && this.timer) {
		clearInterval(this.timer);
		this.timer = null;
	}
	BOARD.draw();
};
Player.prototype.draw = function(context) {
	var x = BOARD.offsetLeft + this.x * BLOCK_SIZE;
	var y = BOARD.offsetTop + this.y * BLOCK_SIZE;
	var tx = BOARD.toBackX(x + BLOCK_SIZE_HALF + this.nx, 1);
	var ty = BOARD.toBackY(y + BLOCK_SIZE_HALF + this.ny, 1);
	context.setStrokeColor("#005000", 1.0);
	context.setLineWidth(1);
	context.setLineCap("round");
	if(x - (BLOCK_SIZE + BLOCK_SIZE_HALF) < BOARD.centerX) {
		context.fillStyle = "#00a000";
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x + BLOCK_SIZE, y);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.lineTo(tx, ty);
		context.closePath();
		context.fill();
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x + BLOCK_SIZE, y);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.lineTo(tx, ty);
		context.closePath();
		context.stroke();
	}
	if(y - (BLOCK_SIZE + BLOCK_SIZE_HALF) < BOARD.centerY) {
		context.fillStyle = "#007000";
		context.beginPath();
		context.moveTo(tx, ty);	
		context.lineTo(x, y + BLOCK_SIZE);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.closePath();
		context.fill();
		context.beginPath();
		context.moveTo(tx, ty);	
		context.lineTo(x, y + BLOCK_SIZE);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.closePath();
		context.stroke();
	}
	if(x + BLOCK_SIZE + BLOCK_SIZE + BLOCK_SIZE_HALF > BOARD.centerX) {
		context.fillStyle = "#009000";
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x, y + BLOCK_SIZE);
		context.lineTo(x, y);
		context.lineTo(tx, ty);
		context.closePath();
		context.fill();
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x, y + BLOCK_SIZE);
		context.lineTo(x, y);
		context.lineTo(tx, ty);
		context.closePath();
		context.stroke();
	}
	if(y + BLOCK_SIZE + BLOCK_SIZE + BLOCK_SIZE_HALF > BOARD.centerY) {
		context.fillStyle = "#00f000";
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x + BLOCK_SIZE, y);
		context.lineTo(x, y);
		context.closePath();
		context.fill();
		context.beginPath();
		context.moveTo(tx, ty);
		context.lineTo(x + BLOCK_SIZE, y);
		context.lineTo(x, y);
		context.closePath();
		context.stroke();
	}
};

function Level(name, fields, playerX, playerY, edit) {
	this.name = name;
	this.fields = fields;
	this.playerX = playerX;
	this.playerY = playerY;
	this.edit = edit;
}

Level.prototype.toString = function() {
	var s = this.name + "|";
	for(var y=0;y<this.fields.length;y++) {
		for(var x=0;x<this.fields[y].length;x++) {
			if(this.fields[y][x])
				s += String(this.fields[y][x]);
			else
				s += "_";
		}
		s += "|";
	}
	s += this.playerX + "|" + this.playerY;
	return s;
};

Level.fromString = function(s) {
	if(typeof s !== "string" || s.length < 10)
		return null;
	var a = s.split("|");
	if(a.length > 12 || a.length < 5) {
		return null;
	}
	var f = new Array();
	for(var i=1;i<a.length-2;i++) {
		if(typeof a[i] !== "string") {
			return null;
		}
		var r = new Array();
		for(var j=0;j<a[i].length;j++) {
			if(a[i].length < 2 || a[i].length > 9) {
				return null;
			}
			var c = a[i].substr(j,1);
			if(c == "_") {
				r[j] = undefined;
			}
			else {
				r[j] = Number(c);
				if(r[j] != B && r[j] != T && r[j] != X && r[j] != W && r[j] != I) {
					return null;
				}
			}
		}
		f[i-1] = r;
	}
	var x = Number(a[a.length-2]);
	var y = Number(a[a.length-1]);
	if(y > f.length - 1 || x > f[y].length - 1 || f[y][x] !== undefined)
		return null;
	var n = String(a[0]);
	while(n.charAt(0) == 32)
		n = n.substring(1);
	if(n.length == 0)
		n = "Unnamed";
	return new Level(n, f, x, y, false);
};

function Marker(x, y) {
	this.x = x;
	this.y = y;
}

Marker.prototype.draw = function(context, depth) {
	var rx1 = BOARD.toBackX(BOARD.offsetLeft + this.x * BLOCK_SIZE, depth);
	var ry1 = BOARD.toBackY(BOARD.offsetTop + this.y * BLOCK_SIZE, depth);
	var rx2 = BOARD.toBackX(BOARD.offsetLeft + this.x * BLOCK_SIZE + BLOCK_SIZE, depth);
	var ry2 = BOARD.toBackY(BOARD.offsetTop + this.y * BLOCK_SIZE + BLOCK_SIZE, depth);
	context.globalAlpha = 0.5;
	context.fillStyle = "#ff8080";
	context.fillRect(rx1, ry1, rx2 - rx1, ry2-ry1);
	context.setStrokeColor("#ff0000", 1.0);
	context.setLineWidth(2);
	context.setLineCap("round");
	context.strokeRect(rx1, ry1, rx2 - rx1, ry2-ry1);
	context.globalAlpha = 1.0;
};

function Board(level) {
	this.timer = null;
	this.setLevel(level);
}

Board.prototype.setLevel = function(level) {
	if(this.timer != null) {
		clearInterval(this.timer);
		this.timer = null;
	}
	this.edit = level.edit;
	this.zoom = 0;
	this.angle = 0;
	this.target = 0;
	this.speed = 0;
	this.maxX = 0;
	this.fields = new Array();
	for(var y=0; y<level.fields.length; y++) {
		var l = level.fields[y].length;
		if(l > this.maxX) this.maxX = l;
		this.fields[y] = new Array();
		for(var x=0; x<l; x++) {
			this.fields[y][x] = this.elementForId(level.fields[y][x]);
		}
	}
	this.centerX = 100;
	this.centerY = 100;
	this.offsetLeft = this.centerX - ((this.edit ? MAX_LEVEL_WIDTH : this.maxX) * BLOCK_SIZE_HALF);
	this.offsetTop = this.centerY - ((this.edit ? MAX_LEVEL_HEIGHT : level.fields.length) * BLOCK_SIZE_HALF);
	this.player = new Player(level.playerX, level.playerY);
	if(this.edit) {
		this.marker = new Marker(level.playerX, level.playerY);
		this.zoom = 0.7;
		this.angle = -10.5;
	}
	this.restoreFocus();
};

Board.prototype.restoreFocus = function() {
	if(this.edit) {
		window.onkeydown = function(e) {
			if(e.keyCode == 39) {
				if(BOARD.marker.x == MAX_LEVEL_WIDTH-1) {
					return;
				}
				BOARD.marker.x++;
				BOARD.draw();
				e.preventDefault();
			}
			else if(e.keyCode == 37) {
				if(BOARD.marker.x == 0) {
					return;
				}
				BOARD.marker.x--;
				BOARD.draw();
				e.preventDefault();
			}
			else if(e.keyCode == 38) {
				if(BOARD.marker.y == 0) {
					return;
				}
				BOARD.marker.y--;
				BOARD.draw();
				e.preventDefault();
			}
			else if(e.keyCode == 40) {
				if(BOARD.marker.y == MAX_LEVEL_HEIGHT-1) {
					return;
				}
				BOARD.marker.y++;
				BOARD.draw();
				e.preventDefault();
			}
			else if(e.keyCode == 84/* t */) {
				if(BOARD.player.x != BOARD.marker.x || BOARD.player.y != BOARD.marker.y) {
					BOARD.fields[BOARD.marker.y][BOARD.marker.x] = new Wall(1.0);
					ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = T;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 88/* x */) {
				if(BOARD.player.x != BOARD.marker.x || BOARD.player.y != BOARD.marker.y) {
					BOARD.fields[BOARD.marker.y][BOARD.marker.x] = new Target();
					ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = X;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 66/* b */) {
				if(BOARD.player.x != BOARD.marker.x || BOARD.player.y != BOARD.marker.y) {
					BOARD.fields[BOARD.marker.y][BOARD.marker.x] = new Box();
					ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = B;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 87/* w */) {
				if(BOARD.player.x != BOARD.marker.x || BOARD.player.y != BOARD.marker.y) {
					BOARD.fields[BOARD.marker.y][BOARD.marker.x] = new Wall(0.5);
					ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = W;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 73/* i */) {
				if(BOARD.player.x != BOARD.marker.x || BOARD.player.y != BOARD.marker.y) {
					BOARD.fields[BOARD.marker.y][BOARD.marker.x] = new Wall(0);
					ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = I;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 80/* p */) {
				if(BOARD.fields[BOARD.marker.y][BOARD.marker.x] == null) {
					BOARD.player.x = BOARD.marker.x;
					BOARD.player.y = BOARD.marker.y;
					ALL_LEVELS[currentLevel].playerX = BOARD.marker.x;
					ALL_LEVELS[currentLevel].playerY = BOARD.marker.y;
					BOARD.draw();
				}
				e.preventDefault();
			}
			else if(e.keyCode == 8/* backspace */) {
				BOARD.fields[BOARD.marker.y][BOARD.marker.x] = null;
				ALL_LEVELS[currentLevel].fields[BOARD.marker.y][BOARD.marker.x] = null;
				BOARD.draw();
				e.preventDefault();
			}
			//else alert(e.keyCode);
		};
	}
	else {
		this.marker = null;
		window.onkeydown = function(e) {
			if(e.keyCode == 39) {
				if(BOARD.speed >= 0)
					BOARD.speed = -ROTATION_SPEED;
				BOARD.target -= 90;
				if(BOARD.target < 0) BOARD.target += 360;
				BOARD.player.turnLeft();
				BOARD.rotate();
				if(ANIMATIONS) {
					if(!BOARD.timer)
						BOARD.timer = setInterval("BOARD.rotate()", 25);
				}
				e.preventDefault();
			}
			else if(e.keyCode == 37) {
				if(BOARD.speed <= 0)
					BOARD.speed = ROTATION_SPEED;
				BOARD.target += 90;
				if(BOARD.target >= 360) BOARD.target -= 360;
				BOARD.player.turnRight();
				BOARD.rotate();
				if(ANIMATIONS) {
					if(!BOARD.timer)
						BOARD.timer = setInterval("BOARD.rotate()", 25);
				}
				e.preventDefault();
			}
			else if(e.keyCode == 38) {
				BOARD.player.move();
				e.preventDefault();
			}
		};
	}
};

Board.prototype.elementForId = function(id) {
	switch(id) {
	case B:
		return new Box();
	case W:
		return new Wall(0.5);
	case T:
		return new Wall(1.0);
	case I:
		return new Wall(0);
	case X:
		return new Target();
	default:
		return null;
	}
}

Board.prototype.draw = function() {
	var context = canvasFront.getContext("2d");

	context.save();
	context.setFillColor(0.1, 0.1, 0.1, 1.0);
	context.fillRect(0, 0, 200, 200);
	context.restore();

	context.save();
	context.translate(100, 100);
	if(this.zoom != 0) {
		context.scale(this.zoom, this.zoom);
		context.rotate((1-this.zoom) * 35 * Math.PI / 180.0);
	}
	context.rotate(this.angle * Math.PI / 180.0);
	if(this.edit) {
		this.cx = (this.marker.x+.5) * BLOCK_SIZE;
		this.cy = (this.marker.y+.5) * BLOCK_SIZE;
	}
	else {
		this.cx = (this.player.x+.5) * BLOCK_SIZE;
		this.cy = (this.player.y+.5) * BLOCK_SIZE;
	}
	context.translate(-this.offsetLeft-this.cx, -this.offsetTop-this.cy);

	if(this.edit) {
		this.marker.draw(context, -0.1);
		context.setStrokeColor("#707070", 1.0);
		context.setLineWidth(3);
		context.setLineCap("round");
		context.strokeRect(this.offsetLeft, this.offsetTop, BLOCK_SIZE*MAX_LEVEL_WIDTH, BLOCK_SIZE*MAX_LEVEL_HEIGHT);
	}
	
	var mhy = Math.ceil(this.fields.length / 2.0);
	var mhx = Math.ceil(this.maxX / 2.0);
	
	// Right-Bottom
	for(var y=this.fields.length-1; y>=mhy; y--) {
		for(var x=this.maxX-1; x>=mhx; x--) {
			if(this.fields[y][x])
				this.fields[y][x].draw(context, x, y);
			if(this.player.x === x && this.player.y === y)
				this.player.draw(context);
		}
	}
	// Left-Bottom
	for(var y=this.fields.length-1; y>=mhy; y--) {
		for(var x=0; x<mhx; x++) {
			if(this.fields[y][x])
				this.fields[y][x].draw(context, x, y);
			if(this.player.x === x && this.player.y === y)
				this.player.draw(context);
		}
	}
	// Right-Top
	for(var y=0; y<mhy; y++) {
		for(var x=this.maxX-1; x>=mhx; x--) {
			if(this.fields[y][x])
				this.fields[y][x].draw(context, x, y);
			if(this.player.x === x && this.player.y === y)
				this.player.draw(context);
		}
	}
	// Left-Top
	for(var y=0; y<mhy; y++) {
		for(var x=0; x<mhx; x++) {
			if(this.fields[y][x])
				this.fields[y][x].draw(context, x, y);
			if(this.player.x === x && this.player.y === y)
				this.player.draw(context);
		}
	}
	if(this.edit) {
		this.marker.draw(context, 1.0);
	}
	context.restore();
	
	context.strokeRect(0, 0, 200, 200);
};

Board.prototype.block = function(context, ix, iy, depth, colors) {
	var x = BOARD.offsetLeft + ix * BLOCK_SIZE;
	var y = BOARD.offsetTop + iy * BLOCK_SIZE;
	var x1 = this.toBackX(x, depth);
	var x2 = this.toBackX(x + BLOCK_SIZE, depth);
	var y1 = this.toBackY(y, depth);
	var y2 = this.toBackY(y + BLOCK_SIZE, depth);
	if(x > this.centerX) {
		context.beginPath();
		context.moveTo(x, y);
		context.lineTo(x1, y1);
		context.lineTo(x1, y2);
		context.lineTo(x, y + BLOCK_SIZE);
		context.closePath();
		context.fillStyle = colors[2];
		context.fill();
		context.beginPath();
		context.moveTo(x, y);
		context.lineTo(x1, y1);
		context.lineTo(x1, y2);
		context.lineTo(x, y + BLOCK_SIZE);
		context.closePath();
		context.setStrokeColor(colors[3], 1.0);
		context.setLineWidth(1);
		context.setLineCap("round");
		context.stroke();
	}
	if(x + BLOCK_SIZE < this.centerX) {
		context.beginPath();
		context.moveTo(x + BLOCK_SIZE, y);
		context.lineTo(x2, y1);
		context.lineTo(x2, y2);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.lineTo(x + BLOCK_SIZE, y);
		context.closePath();
		context.fillStyle = colors[4];
		context.fill();
		context.beginPath();
		context.moveTo(x + BLOCK_SIZE, y);
		context.lineTo(x2, y1);
		context.lineTo(x2, y2);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.closePath();
		context.setStrokeColor(colors[5], 1.0);
		context.setLineWidth(1);
		context.setLineCap("round");
		context.stroke();
	}
	if(y > this.centerY) {
		context.beginPath();
		context.moveTo(x, y);
		context.lineTo(x1, y1);
		context.lineTo(x2, y1);
		context.lineTo(x + BLOCK_SIZE, y);
		context.lineTo(x, y);
		context.closePath();
		context.fillStyle = colors[6];
		context.fill();
		context.beginPath();
		context.moveTo(x, y);
		context.lineTo(x1, y1);
		context.lineTo(x2, y1);
		context.lineTo(x + BLOCK_SIZE, y);
		context.closePath();
		context.setStrokeColor(colors[7], 1.0);
		context.setLineWidth(1);
		context.setLineCap("round");
		context.stroke();
	}
	if(y + BLOCK_SIZE < this.centerY) {
		context.beginPath();
		context.moveTo(x, y + BLOCK_SIZE);
		context.lineTo(x1, y2);
		context.lineTo(x2, y2);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.lineTo(x, y + BLOCK_SIZE);
		context.closePath();
		context.fillStyle = colors[8];
		context.fill();
		context.beginPath();
		context.moveTo(x, y + BLOCK_SIZE);
		context.lineTo(x1, y2);
		context.lineTo(x2, y2);
		context.lineTo(x + BLOCK_SIZE, y + BLOCK_SIZE);
		context.closePath();
		context.setStrokeColor(colors[9], 1.0);
		context.setLineWidth(1);
		context.setLineCap("round");
		context.stroke();
	}
	context.fillStyle = colors[0];
	context.fillRect(x1, y1, x2-x1, y2-y1);
	context.setStrokeColor(colors[1], 1.0);
	context.setLineWidth(1.5);
	context.setLineCap("round");
	context.strokeRect(x1, y1, x2-x1, y2-y1);
};

var D_FACT = 0.3;
Board.prototype.toBackX = function(x, depth) {
	return x - ((100 - x) * depth * D_FACT);
};
Board.prototype.toBackY = function(y, depth) {
	return y - ((100 - y) * depth * D_FACT);
};

Board.prototype.rotate = function() {
	this.angle += this.speed;
	if(this.angle >= 360) this.angle -= 360;
	if(this.angle < 0) this.angle += 360;
	if(this.target == this.angle) {
		clearInterval(this.timer);
		this.angle = this.target;
		this.speed = null;
		this.timer = null;
	}
	this.draw();
};

function initGame(level) {
	BOARD = new Board(level);
	BOARD.draw();
}



